#
# Copyright (c) 2023 Wind River Systems, Inc.
#
# SPDX-License-Identifier: Apache-2.0
#
# All Rights Reserved.
#

""" System inventory App lifecycle operator."""

from k8sapp_intel_device_plugins_operator.common import constants as app_constants
from oslo_log import log as logging
from sysinv.common import constants
from sysinv.common import exception
from sysinv.helm import lifecycle_base as base
from sysinv.helm import lifecycle_utils
import yaml

LOG = logging.getLogger(__name__)


class IntelDevicePluginsAppLifecycleOperator(base.AppLifecycleOperator):
    """Class for app lifecycle management of app-intel-device-plugins."""

    def app_lifecycle_actions(self, context, conductor_obj, app_op, app, hook_info):
        """
        Perform lifecycle actions for an operation.

        :param context: request context, can be None
        :param conductor_obj: conductor object, can be None
        :param app_op: AppOperator object
        :param app: AppOperator.Application object
        :param hook_info: LifecycleHookInfo object

        """
        # Semantic Check
        if (hook_info.lifecycle_type == constants.APP_LIFECYCLE_TYPE_SEMANTIC_CHECK
                and hook_info.operation == constants.APP_APPLY_OP
                and hook_info.relative_timing == constants.APP_LIFECYCLE_TIMING_PRE):
            return self._pre_semantic_check(app, app_op)

        if hook_info.lifecycle_type == constants.APP_LIFECYCLE_TYPE_FLUXCD_REQUEST:
            if hook_info.operation == constants.APP_APPLY_OP:
                if hook_info.relative_timing == constants.APP_LIFECYCLE_TIMING_POST:
                    lifecycle_utils.create_local_registry_secrets(app_op, app, hook_info)
                    lifecycle_utils.add_pod_security_admission_controller_labels(
                        app_op, app, hook_info)
                    return self.add_component_label_in_pods(app_op, app)
            elif hook_info.operation == constants.APP_REMOVE_OP and \
                    hook_info.relative_timing == constants.APP_LIFECYCLE_TIMING_POST:
                return lifecycle_utils.delete_local_registry_secrets(app_op, app, hook_info)

        super(IntelDevicePluginsAppLifecycleOperator, self).app_lifecycle_actions(
            context, conductor_obj, app_op, app, hook_info
        )

    def _pre_semantic_check(self, app, app_op):
        LOG.debug(f"Executing pre_semantic_check for {app.name} app")

        dbapi = app_op._dbapi
        try:
            nfd_kube_app = dbapi.kube_app_get(app_constants.HELM_APP_NFD)
            LOG.info("Node Feature Discovery (NFD) Application found - "
                     f"Version: {nfd_kube_app.app_version} - "
                     "Status: {nfd_kube_app.status}")

            nfd_installed = nfd_kube_app.status == constants.APP_APPLY_SUCCESS

        except exception.KubeAppNotFound:
            nfd_installed = False

        if not nfd_installed:
            if self._is_nfd_required(app_op):
                raise exception.LifecycleSemanticCheckException(
                    "Node Feature Discovery (NFD) Application is required. "
                    "You can bypass this check by setting the "
                    f"{app_constants.HELM_NFD_REQUIRED_PARAM} parameter to "
                    "False using overrides.")

            LOG.info("Bypass flag for Node Feature Discovery (NFD) "
                     "Application found.")

    def _is_nfd_required(self, app_op):

        """Checks the state of the parameter that controls whether the NFD
        application is required

        :param app_op: app_op
        :return True if enabled otherwise False
        """
        val = True

        try:
            dbapi_instance = app_op._dbapi
            app = dbapi_instance.kube_app_get(
                app_constants.HELM_APP_INTEL_DEVICE_PLUGINS_OPERATOR)
            db_app_id = dbapi_instance.kube_app_get(app.name).id
            # user_overrides = self._get_user_overrides(app, dbapi)
            charts = self._get_charts_enabled(dbapi_instance, db_app_id)

            for chart in charts:
                # Loading user-overrides
                user_overrides = chart['user_overrides']
                if user_overrides and app_constants.HELM_NFD_REQUIRED_PARAM in user_overrides:
                    if isinstance(user_overrides[app_constants.HELM_NFD_REQUIRED_PARAM], bool):
                        val = user_overrides[app_constants.HELM_NFD_REQUIRED_PARAM]

                    LOG.error(f"The value of parameter "
                          f"{app_constants.HELM_NFD_REQUIRED_PARAM} must be "
                          "true or false.")

        except exception.KubeAppNotFound as e:
            LOG.error("Failed to access app info "
                      f"{app_constants.HELM_APP_INTEL_DEVICE_PLUGINS_OPERATOR}: "
                      f"{e}")

        except exception.HelmOverrideNotFound as e:
            LOG.error("Failed to access user overrides from chart "
                      f"{app_constants.HELM_CHART_INTEL_DEVICE_PLUGINS_OPERATOR}: "
                      f"{e}")

        return val

    def add_component_label_in_pods(self, app_op, app):
        """
        Adding component label in pods.

        :param app_op: AppOperator object
        :param app: AppOperator.Application object

        """

        dbapi_instance = app_op._dbapi
        db_app_id = dbapi_instance.kube_app_get(app.name).id

        component_constant = app_constants.HELM_COMPONENT_LABEL_INTEL_DEVICE_PLUGINS_OPERATOR

        # List all charts enabled
        charts = self._get_charts_enabled(dbapi_instance, db_app_id)

        for chart in charts:
            # Loading user-overrides
            user_overrides = chart['user_overrides']
            # If user-overrides exists, checking if label was set by user.
            if user_overrides and component_constant in user_overrides:
                dict_chart_overrides = yaml.safe_load(user_overrides)
                label = dict_chart_overrides[component_constant]
                # Checking if it's a supported label. If not, set platform as default label.
                if label not in app_constants.HELM_COMPONENT_SUPPORTED_LABELS:
                    dict_chart_overrides[component_constant] = 'platform'
                    LOG.warn(f'User override for core affinity {label} is not supported,' +
                              'using platform as default label.')

                    chart['user_overrides'] = yaml.safe_dump(dict_chart_overrides)

                    dbapi_instance.helm_override_update(
                        db_app_id,
                        chart['name'],
                        app_constants.HELM_NS_INTEL_DEVICE_PLUGINS_OPERATOR,
                        chart)

    def _get_charts_enabled(self, dbapi_instance, db_app_id):
        # Listing all helm charts from db_app_id
        overrides = dbapi_instance.helm_override_get_all(
            app_id=db_app_id
        )
        # Getting only charts enabled
        charts = [{
                    "name": i.name,
                    "user_overrides": i.user_overrides,
                    "system_overrides": i.system_overrides
                  } for i in overrides if i.system_overrides['enabled'] is True]
        return charts
